# fit (framework for interactive testing) support

import sys, os, string
from OFS.DTMLDocument import DTMLDocument
from Utils import Popen3

# from tests/support.py:
def pdir(path): return os.path.split(path)[0]
ZWIKIDIR = pdir(os.path.abspath(__file__))

try:
    from fit.Parse import Parse
    from fit.Fixture import Fixture
    class FitSupport:
        """
        Mix-in class for fit support
        """
        # no good, doesn't override
        def _render(self, client=None, REQUEST={}, RESPONSE=None, **kw):
            """
            Hack the render method to always run fit as the last step
            """
            method = self._renderMethod()
            t = apply(method,(self, REQUEST, RESPONSE), kw)
            t = Fixture().doTables(Parse(t))
            return t

        def render_dtmlfithtml(self, client=None, REQUEST={}, RESPONSE=None, **kw):
            """
            render the page as DTML + fit tests + HTML
            """
            # pre render
            t = str(self.read())
            if not self._prerendered:
                get_transaction().note('prerender')
                self._prerendered = t or '\n'
                self.cook()
            if kw.get('pre_only',0): return
            # final render
            t = apply(DTMLDocument.__call__,(self,client,REQUEST,RESPONSE),kw)
            t = self.runFitOn(t)
            t = apply(self.addStandardLayoutTo,(t,),kw)
            return t

        def render_stxdtmlfithtml(self, client=None, REQUEST={},
                                   RESPONSE=None, **kw):
            """
            render the page as structured text + DTML + fit tests + HTML
            """
            # pre render
            t = str(self.read())
            if not self._prerendered:
                get_transaction().note('prerender')
                t = self.stxToHtml(t)
                self._prerendered = t or '\n'
                self.cook()
            if kw.get('pre_only',0): return
            # final render
            t = apply(DTMLDocument.__call__,(self, client, REQUEST, RESPONSE), kw)
            t = self.runFitOn(t)
            t = apply(self.addStandardLayoutTo,(t,),kw)
            return t

        def _runFitInternallyOn(self,text):
            fitTables = Parse(text)

            # reset class-variable counts
            # not working
            Fixture.rights = 0
            Fixture.wrongs = 0
            Fixture.ignores = 0
            Fixture.exceptions = 0
            Fixture.summary = {}

            # find fit in ZWiki product directory as well as PYTHONPATH
            oldpath = sys.path
            sys.path.insert(0,ZWIKIDIR)

            Fixture().doTables(fitTables)

            sys.path = oldpath
            return str(fitTables)

        def _runFitExternallyOn(self,text):
            oldpath = sys.path
            sys.path.insert(0,ZWIKIDIR)
            p = Popen3(
                '''PYTHONPATH=%s python -c \
                     "import fit.FileRunner; \
                      fit.FileRunner.FileRunner(('','-','-')).run()"''' % \
                  string.join(sys.path,':'),
                input=text)
            sys.path = oldpath
            return p.out

        runFitOn = _runFitInternallyOn

except ImportError:
    class FitSupport:
        pass
